package de.hdm_stuttgart.battlearena.Controller;

import de.hdm_stuttgart.battlearena.Controller.Utilities.CreateMediaPlayer;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Exceptions.DatabaseException;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Persistence;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.StartupThread;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Cursor;
import javafx.scene.layout.BorderPane;
import javafx.scene.media.MediaPlayer;
import javafx.scene.media.MediaView;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Objects;
import java.util.ResourceBundle;

public class IntroController implements Initializable {

    @FXML
    private MediaView mediaView;
    @FXML
    private BorderPane introParent;
    private final String fileName = "src/main/resources/videos/";
    //    nextVideo() will iterate through this array so the next video will be played TODO: change the videos to non memes lmao
    private final String[] videos = {"sony.mp4", "gamecube.mp4", "gameboy.mp4", "monke.mp4"};
    private File file = new File(fileName + videos[0]);
    private int counter = 0;
    private MediaPlayer mediaPlayer;
    private final Persistence persistence = Persistence.getInstance();
    private static final Logger log = LogManager.getLogger(Persistence.class);

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        introParent.setStyle("-fx-background-color: black;");
        introParent.setCursor(Cursor.NONE);
        createMediaPlayer();
        new StartupThread().run();
    }

    private void createMediaPlayer() {
//        initializing this.mediaPlayer
        mediaPlayer = new CreateMediaPlayer().getMediaPlayer(mediaView, file, true, introParent);
        mediaView.setMediaPlayer(mediaPlayer);

        mediaPlayer.setOnEndOfMedia(() -> {
            if (counter == videos.length - 1) {
                try {
                    videoEnd();
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            } else {
                nextVideo();
            }
        });

//        TODO: make any button pressed work
        introParent.setOnMouseClicked((mouseEvent) -> {
            if (counter == videos.length - 1) {
                try {
                    videoEnd();
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            } else {
                nextVideo();
            }
        });
    }

    private void videoEnd() throws IOException {
        boolean isStartUp = true, noAccount = false, statsLoaded = false;
        while (isStartUp) {
            try {
                persistence.loadPlayerAccount();
                persistence.verifyPlayerAccount();
                mediaPlayer.dispose();
                isStartUp = false;
            } catch (DatabaseException e) {
                noAccount = true;
//            TODO: reset player account if account fails to load in loadPlayerAccount()
            }
            if (noAccount) {
                introParent.getScene().setRoot(FXMLLoader.load(Objects.requireNonNull(getClass().getResource("/fxml/PlayerCreateScene.fxml"))));
            }
        }
        while (!statsLoaded) { // while loop because Martin said so
            try {
                persistence.loadPlayerStatistics();
                statsLoaded = true;
            } catch (DatabaseException e) {
                log.error(e);
            }
        }
        introParent.getScene().setRoot(FXMLLoader.load(Objects.requireNonNull(getClass().getResource("/fxml/MenuBorderPane.fxml"))));
    }

    private void nextVideo() {
        mediaPlayer.dispose();
        file = new File(fileName + videos[counter + 1]);
        counter++;
        createMediaPlayer();
    }
}
