package de.hdm_stuttgart.battlearena.Model.DataStorage.Classes;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Exceptions.GSONException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.FileReader;
import java.io.FileWriter;
import java.lang.reflect.Type;
import java.util.ArrayList;

public class GsonHandler {
    Gson gson = new GsonBuilder().setPrettyPrinting().create();

    private static final Logger log = LogManager.getLogger(GsonHandler.class);
    Type mapDataType = new TypeToken<ArrayList<MapData>>(){}.getType();
    Type playerStatsType = new TypeToken<PlayerStatistics>(){}.getType();
    Type settingsType = new TypeToken<AppSettings>(){}.getType();
    Type accountType = new TypeToken<PlayerAccount>(){}.getType();

    private final String filePathCoreMaps = "src/main/resources/maps/coreMaps.json";
    private final String filePathCommunityMaps = "src/main/resources/maps/communityMaps.json";
    private final String filePathPlayerStats = "src/main/resources/player/playerStatsLocal.json";
    private final String filePathPlayerAccount = "src/main/resources/player/playerAccount.json";
    private final String filePathSettings = "src/main/resources/player/appSettings.json";

    public ArrayList<MapData> loadMaps(MapType type) throws GSONException {
        String filePath;
        if(type == MapType.COREMAP){
            filePath = filePathCoreMaps;
        }
        else{
            filePath = filePathCommunityMaps;
        }
        try (FileReader reader = new FileReader(filePath)) {
            ArrayList<MapData> maps = gson.fromJson(reader, mapDataType);
            log.info("GSON - Maps successfully loaded from JSON!");
            return maps;
        } catch (Exception e) {
            log.info(e);
            if(type == MapType.COREMAP){
                log.info("GSON - Loading Core-Maps from JSON failed!");
                throw new GSONException("Loading Core-Maps from JSON failed!");
            }
            else{
                log.info("GSON - Loading Community-Maps from JSON failed!");
                throw new GSONException("Loading Community-Maps from JSON failed!");
            }
        }
    }

    public void saveMaps(ArrayList<MapData> maps, MapType type) throws GSONException {
        String filePath;
        if(type == MapType.COREMAP){
            filePath = filePathCoreMaps;
        }
        else{
            filePath = filePathCommunityMaps;
        }
        try (FileWriter writer = new FileWriter(filePath)) {
            gson.toJson(maps, writer);
            log.info("GSON - Maps successfully saved to JSON!");
        } catch (Exception e) {
            log.info(e);
            if(type == MapType.COREMAP){
                log.info("GSON - Saving Core-Maps to JSON failed!");
                throw new GSONException("Saving Core-Maps to JSON failed!");
            }
            else{
                log.info("GSON - Saving Community-Maps to JSON failed!");
                throw new GSONException("Saving Community-Maps to JSON failed!");
            }
        }
    }

    public PlayerStatistics loadStats() throws GSONException {
        try (FileReader reader = new FileReader(filePathPlayerStats)) {
            PlayerStatistics stats = gson.fromJson(reader, playerStatsType);
            log.info("GSON - Player statistics successfully loaded from JSON!");
            return stats;
        } catch (Exception e) {
            log.info(e);
            log.info("GSON - Loading player statistics from JSON failed!");
            throw new GSONException("Loading player statistics from JSON failed!");
        }
    }

    public void saveStats(PlayerStatistics stats) throws GSONException {
        try (FileWriter writer = new FileWriter(filePathPlayerStats)) {
            gson.toJson(stats, writer);
            log.info("GSON - Player statistics successfully saved to JSON!");
        } catch (Exception e) {
            log.info(e);
            log.info("GSON - Saving player statistics to JSON failed!");
            throw new GSONException("Saving player statistics to JSON failed!");
        }
    }

    public AppSettings loadSettings() throws GSONException {
        try (FileReader reader = new FileReader(filePathSettings)) {
            AppSettings settings = gson.fromJson(reader, settingsType);
            log.info("GSON - Settings successfully loaded from JSON!");
            return settings;
        } catch (Exception e) {
            log.info(e);
            log.info("GSON - Loading settings from JSON failed!");
            throw new GSONException("Loading settings from JSON failed!");
        }
    }

    public void saveSettings(AppSettings settings) throws GSONException {
        try (FileWriter writer = new FileWriter(filePathSettings)) {
            gson.toJson(settings, writer);
            log.info("GSON - Settings successfully saved to JSON!");
        } catch (Exception e) {
            log.info(e);
            log.info("GSON - Saving settings to JSON failed!");
            throw new GSONException("Saving settings to JSON failed!");
        }
    }

    public PlayerAccount loadAccount() throws GSONException {
        try (FileReader reader = new FileReader(filePathPlayerAccount)) {
            PlayerAccount account = gson.fromJson(reader, accountType);
            log.info("GSON - Player account information successfully loaded from JSON!");
            return account;
        } catch (Exception e) {
            log.info(e);
            log.info("GSON - Loading player account information from JSON failed!");
            throw new GSONException("Loading player account information from JSON failed!");
        }
    }

    public void saveAccount(PlayerAccount account) throws GSONException {
        try (FileWriter writer = new FileWriter(filePathPlayerAccount)) {
            gson.toJson(account, writer);
            log.info("GSON - Player account information successfully saved to JSON!");
        } catch (Exception e) {
            log.info(e);
            log.info("GSON - Saving player account information to JSON failed!");
            throw new GSONException("Saving player account information to JSON failed!");
        }
    }


}
